//---------------------------------------------------------------------------

//#pragma hdrstop

//---------------------------------------------------------------------------

//#pragma argsused

#include <stdio.h>
#include <stdint.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

#include "defs.h"
#include "print.h"
#include "serial.h"
#include "avroo.h"

const char* portname 		= NULL; //COM_DEFAULT;
const char* portconfig 		= NULL; //CFG_DEFAULT;
const char* trigger 		= "<rest>";
const char* flashfile 		= NULL;
const char* eepfile 		= NULL;

//bool do_interact 			= false;

void usage(const char* progname)
{
	printf_yellow("usage: \r\n");
	printf("%s -i\r\n", progname);	
	printf("%s -p COMPORT [-c PORTCONFIG] [-f FLASHFILE] [-e EEPROMFILE] [-t TRIGGER]\r\n", progname);
	printf("  COMPORT: e.g. " COM_DEFAULT " ... \r\n");	
	printf("  PORTCONFIG: e.g. \"" CFG_DEFAULT "\" \r\n");
	printf("  FLASHFILE: a Intel Hex (.hex or .eep) or Binary Image File (.bin) \r\n");
	printf("  EEPROMFILE: a Intel Hex (.eep) or Binary Image File (.bin) \r\n");
	printf("  TRIGGER: Triggerstring for the Bootloader to launch. \r\n");
}

bool scanargs(int argc, char** argv)
{
	int argp = 1;

	for (int i = 1; i < argc; i++)
	{
		char* arg = argv[i];

		if (STREQU(arg, "-p") || STREQU(arg, "--portname"))
		{
			portname = argv[i+1];
			i++;
			argp += 2;
		}
		else if (STREQU(arg, "-c") || STREQU(arg, "--portconfig"))
		{
			portconfig = argv[i+1];
			i++;
			argp += 2;
		}
		else if (STREQU(arg, "-t") || STREQU(arg, "--trigger"))
		{
			trigger = argv[i+1];
			i++;
			argp += 2;
		}
		else if (STREQU(arg, "-f") || STREQU(arg, "--flashfile"))
		{
			flashfile = argv[i+1];
			i++;
			argp += 2;
		}
		else if (STREQU(arg, "-e") || STREQU(arg, "--eepfile"))
		{
			eepfile = argv[i+1];
			i++;
			argp += 2;
		}
		else if (STREQU(arg, "-pf") || STREQU(arg, "--pagesize-flash"))
		{
			g_pagesize_flash = atoi(argv[i+1]);
			i++;
			argp += 2;
		}
		else if (STREQU(arg, "-pe") || STREQU(arg, "--pagesize-eeprom"))
		{
			g_pagesize_eeprom = atoi(argv[i+1]);
			i++;
			argp += 2;
		}
		/*
		else if (STREQU(arg, "-i") || STREQU(arg, "--interactive"))
		{
			interact();
			argp += 1;
		}
		*/
		else if (STREQU(arg, "-h") || STREQU(arg, "--help"))
		{
			usage(argv[0]);
			exit(0);
			return false;
		}
		else if (STREQU(arg, "-v") || STREQU(arg, "--version"))
		{
			printf("Version: " COMPILE_TIMESTAMP "\r\n");
			exit(0);
			return false;
		}		
		else
		{
			eprintf("Error: Unrecognized command line option \"%s\".\r\n", arg);
			usage(argv[0]);
			return false;
		}
	}

	return (argc == argp);
}

/*
bool interact(void)
{
	static char efn_buf[FILENAME_MAX];
	static char ffn_buf[FILENAME_MAX];
	static char cpn_buf[FILENAME_MAX];

	if (portname == NULL)
	{
		printf("Serial Ports in System:\r\n");
		serial_enumports();
		printf("Select Serial Port: ");
		scanf("%31s", cpn_buf);

		if (isdigit(*cpn_buf))
		{
			int num = atoi(cpn_buf);
			snprintf(cpn_buf, sizeof(cpn_buf), "COM%u", num);
		}

		portname = cpn_buf;
	}

	if (flashfile == NULL)
	{
		flashfile = openfiledialog("Select Flash File", ffn_buf, sizeof(ffn_buf));
	}

	if (eepfile == NULL)
	{
		eepfile = openfiledialog("Select EEPROM File", efn_buf, sizeof(efn_buf));
	}
}
*/

int main(int argc, char* argv[])
{
	printf_init();

#if (defined _WIN32)
	printf("avrootloader (_WIN32): start...\r\n");
#elif (defined LINUX)
	printf("avrootloader (LINUX): start...\r\n");	
#endif

	if (!scanargs(argc, argv))
	{
		return 1;
	}

	/*	
	if (do_interact)
	{	
		
	}
	
	printf_green("avrootloader - serial settings: \r\n");	
	printf("- Portname  : %s \r\n", portname);
	printf("- Portconfig: %s \r\n", portconfig);	
	
	waitkeypress("Hit any Key to start...\r\n");	
	
	
	if (serial_open(portname, portconfig))
	{
		char rxbuf[64];
		
		serial_puts("<rtcg>");
		for (int x=0; x<20; x++)
		{
			int rxcnt = serial_readline(rxbuf, sizeof(rxbuf) - 1, 5000, '>');
			if (rxcnt > 0) { rxbuf[rxcnt] = '\0'; printf("Rx: %s\r\n", rxbuf); }
		}
		
		serial_close();
	}
	*/	

	printf("avrootloader - upload settings: \r\n");	
	printf("- Portname  : %s \r\n", portname);
	printf("- Portconfig: %s \r\n", portconfig);	
	printf("- Flashfile : %s \r\n", flashfile);	
	printf("- Eepromfile: %s \r\n", eepfile);		
	printf("- Trigger   : %s \r\n", trigger);
	
	/*
	if (do_interact)
	{	
		waitkeypress("Hit any Key to start...\r\n");	
	}
	*/
	
	bool res = avroo_upload(
		portname, portconfig,
		trigger, strlen(trigger),
		flashfile,
		eepfile
	);

	//waitkeypress("Hit any Key to exit...\r\n");

	printf_deinit();

	return res ? 0 : 1;
}
