//---------------------------------------------------------------------------

#pragma hdrstop

#include "print.h"
//---------------------------------------------------------------------------

#include <ctype.h>
#include <string.h>

#if (defined _WIN32)

#include <conio.h>
#include <windows.h>
HANDLE g_hConsoleOutput = NULL;

void printf_init(void)
{
	g_hConsoleOutput = GetStdHandle(STD_OUTPUT_HANDLE);
	setvbuf(stdin,  NULL, _IONBF, 0);
	setvbuf(stdout, NULL, _IONBF, 0);
}

void printf_color(int color, const char *fmt, ...)
{
	const char *p;
	va_list argp;
	va_start(argp, fmt);
	SetConsoleTextAttribute(g_hConsoleOutput, color);
	vfprintf(stdout, fmt, argp);
	SetConsoleTextAttribute(g_hConsoleOutput, CONSOLE_COLOR_NORMAL);	
	va_end(argp);
}

void printf_deinit(void)
{
	CloseHandle(g_hConsoleOutput);
	g_hConsoleOutput = NULL;
}
#endif

#if (defined LINUX)

void printf_init(void)
{
	setvbuf(stdin,  NULL, _IONBF, 0);
	setvbuf(stdout, NULL, _IONBF, 0);
}

#include <unistd.h>
void sleep_ms(unsigned int ms)
{
	usleep(1000 * ms);
}

unsigned long GetTickCount_ms(void)
{
	struct timespec tp;
	return clock_gettime(CLOCK_MONOTONIC, &tp) ? 0 :
	(unsigned long)((tp.tv_sec * 1000) + (tp.tv_nsec / 1000000));
}

#include <fcntl.h>
#include <termios.h>

int kbhit(void)
{
	struct termios oldt, newt;
	int ch;
	int oldf;

	tcgetattr(STDIN_FILENO, &oldt);
	newt = oldt;
	newt.c_lflag &= ~(ICANON | ECHO);
	tcsetattr(STDIN_FILENO, TCSANOW, &newt);
	oldf = fcntl(STDIN_FILENO, F_GETFL, 0);
	fcntl(STDIN_FILENO, F_SETFL, oldf | O_NONBLOCK);

	ch = getchar();

	tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
	fcntl(STDIN_FILENO, F_SETFL, oldf);

	if(ch != EOF)
	{
		//ungetc(ch, stdin);
		return 1;
	}

	return 0;
}
#endif

void printhex(const char* buf, int len)
{
	for (int i=0; i<len; i++)
	{
		char c = buf[i];
		printf("%02hx ", 0xff & c);
	}
}

void printstr(const char* buf, int len)
{
	for (int i=0; i<len; i++)
	{
		char c = buf[i];
		if (!isprint(c) || c=='\\')
		{
			printf("%02hx ", 0xff & c);
		}
		else
		{
			printf("%c", c);
		}
	}
}

void hexdump(unsigned int addr, const char* buf, int len)
{
	if (len < 0) return;

	for (int i=0; i<len; i+=HEXDUMP_LINELEN)
	{
		printf("%04x | ", addr + i);

		for (int j=0; j<HEXDUMP_LINELEN; j++)
		{
			if (j==HEXDUMP_LINELEN/2) printf(" ");

			if (i+j<len)
			{
				char c = buf[i+j];
				printf("%c", isprint(c) ? c : '.');
			}
			else
			{
				printf(" ");
			}
		}

		printf(" | ");

		for (int j=0; j<HEXDUMP_LINELEN && i+j<len; j++)
		{
			if (j==HEXDUMP_LINELEN/2) (" ");

			if (i+j<len)
			{
				char c = buf[i+j];
				printf("%02hx ", 0xff & c);
			}
			else
			{
				printf(" ");
			}


		}

		printf("\n");
	}
}

void waitkeypress(const char* prompt)
{
	printf("%s", prompt);
	char c;
	while ((c = getchar()) != '\n' && c != EOF);
}

char* timestr(time_t timestamp)
{
	struct tm * timeinfo = localtime(&timestamp);
	static char tstr[24];
	strftime(tstr, sizeof(tstr), "%H:%M:%S", timeinfo);
	return tstr;
}

char wheel(void)
{
	const char* str = "/-\\|";
	static int n = 0;

	n++;
	n %= 4;

	return str[n];
}

char* openfiledialog(const char* prompt, char* buf, int len)
{
#if (defined _WIN32)
	// prompt for filename!
	OPENFILENAME ofn;       // common dialog box structure
	// Initialize OPENFILENAME
	ZeroMemory(&ofn, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = NULL;
	ofn.lpstrFile = buf;
	// Set lpstrFile[0] to '\0' so that GetOpenFileName does not 
	// use the contents of szFile to initialize itself.
	ofn.lpstrFile[0] = '\0';
	ofn.nMaxFile = len;

	ofn.lpstrFilter =
		"Intel-Hex-Files (*.hex;*.eep)\0*.hex;*.eep\0"
		"Binary Files (*.bin)\0*.bin\0"
		"All Files\0(*.*)\0"
		"\0";
	ofn.nFilterIndex = 1;
	ofn.lpstrFileTitle = NULL;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = NULL;
	ofn.lpstrTitle = prompt;
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

	// Display the Open dialog box. 

	printf("%s ...\n", prompt);

	if (GetOpenFileName(&ofn))
	{
		printf("Selected: %s\n", buf);
		return buf;
	}
	else
	{
		printf("Canceled.\n");
		return NULL;
	}
#elif (defined LINUX)
	printf("%s ...\n", prompt);
	fgets(buf, len, stdin);
	return strlen(buf) ? buf : NULL;
#endif
}

/*

void set_console_title(const char *fmt, ...)
{
	const char *p;
	va_list argp;
	char str[256];

	va_start(argp, fmt);

	vsnprintf(str, sizeof(str), fmt, argp);
	SetConsoleTitle(str);

	va_end(argp);
}

void perror(const char* msg)
{
	fprintf(stderr, "Error: %s: %s", msg, strerror(GetLastError()));
}

*/

